"""Admin route modules."""

from . import (
    a2a,
    agents,
    async_routes,
    audit,
    config,
    llm,
    logging,
    logs,
    mcp,
    memory,
    overlord,
    scheduler,
    secrets,
)

__all__ = [
    "a2a",
    "agents",
    "async_routes",
    "audit",
    "config",
    "llm",
    "logging",
    "logs",
    "mcp",
    "memory",
    "overlord",
    "scheduler",
    "secrets",
]
